/* 2009.03.06                */
/* CH375 AVR}CRTv */
/* FAT12/FAT16/FAT32 T|[g*/

/* AVR-GCC -v -mmcu=atmega64 -O2 -xc CH375HFT.c -oCH375HFT.ELF -lCH375HFD */
/* AVR-OBJCOPY -j .text -j .data -O ihex CH375HFT.ELF CH375HFT.HEX */

#include <string.h>
#include <stdio.h>
#include <avr/io.h>

#include<avr/eeprom.h>

#include<avr/pgmspace.h>

#define FOSC   F_CPU
#define F_CPU  7372800
#define BAUD   9600

#define MYUBRR FOSC/BAUD/16-1


int USART_putchar(char CH)
{
	/* Wait for empty transmit buffer */
	while ( !( UCSR0A & (1<<UDRE0)) );
	/* Put data into buffer, sends the data */
	UDR0=CH;
	return 0;
}


int USART_getchar(void)
{
 	/* Wait for data to be received */
	while ( !(UCSR0A & (1<<RXC0)) );
	/* Get and return received data from buffer */
	return UDR0;
}


void COM_Initial(unsigned int UBRR_val)
{
	/* Set baud rate */
	UBRR0H = (unsigned char)(UBRR_val>>8);
	UBRR0L = (unsigned char)UBRR_val;
	/* Enable receiver and transmitter */
	UCSR0B = (1<<RXEN0)|(1<<TXEN0);
	/* Set frame format: 8data, 2stop bit */
	//UCSRC = (1<<URSEL)|(1<<USBS)|(3<<UCSZ0);
	fdevopen(USART_putchar,USART_getchar);//
}


#define LIB_CFG_FILE_IO			1
#define LIB_CFG_INT_EN			0

#define DISK_BASE_BUF_LEN		2048
#define FILE_DATA_BUF_LEN		0x0200

#define CH375_INT_WIRE			( PING & 0x01 )

#define NO_DEFAULT_CH375_F_ENUM		1
#define NO_DEFAULT_CH375_F_QUERY	1

#include "..\CH375HFD.H"

#define CH375_INT			0

#define CH375_AO			1
#define CH375_CS			2
#define CH375_RD			3
#define CH375_WR			4



void mDelay1uS( )
{
	UINT8	count = 2;
	__asm__ volatile (
		"1: dec %0" "\n\t"
		"brne 1b"
		: "=r" (count)
		: "0" (count)
	);
}

void CH375_PORT_INIT( )
{
	DDRD = 0x00;
	PORTG |= (0x01 << CH375_CS);
	PORTG |= (0x01 << CH375_WR);
	PORTG |= (0x01 << CH375_RD); 
	DDRG = 0x1e;  
}

void xWriteCH375Cmd( UINT8 mCmd )		/* CH375CuɎgpR}hCg */
{
	mDelay1uS( ); mDelay1uS( );
	PORTG |= (0x01 << CH375_AO);
	
	PORTD = mCmd;
	DDRD = 0xFF;
	
	PORTG &= ~(0x01 << CH375_CS);
	PORTG &= ~(0x01 << CH375_WR);
	DDRD = 0xFF;  /* delay 100nS */
	
	PORTG |= (0x01 << CH375_CS);
	PORTG |= (0x01 << CH375_WR);
	PORTG |= (0x01 << CH375_RD); 
	
	DDRD = 0x00;
	
	PORTG &= ~(0x01 << CH375_AO);
	mDelay1uS( ); mDelay1uS( );
}

void xWriteCH375Data( UINT8 mData )		/* CH375CuɎgpf[^Cg */
{
	PORTD = mData;
	DDRD = 0xFF;
	PORTG &= ~(0x01 << CH375_AO); 
	PORTG &= ~(0x01 << CH375_CS);
	PORTG &= ~(0x01 << CH375_WR);
	
	DDRD = 0xFF;  /* delay 100nS */
	PORTG |= (0x01 << CH375_CS);
	PORTG |= (0x01 << CH375_WR);
	PORTG |= (0x01 << CH375_RD); 
	
	DDRD = 0x00;
	mDelay1uS( );
}

UINT8 xReadCH375Data( void )			/* CH375CuɎgpf[^[h */
{
	UINT8	mData;
	mDelay1uS( );
	DDRD = 0x00;
	PORTG &= ~(0x01 << CH375_AO); 
	PORTG &= ~(0x01 << CH375_CS);
	PORTG &= ~(0x01 << CH375_RD);
	DDRD = 0x00;  /* delay 100nS */
	mData = PIND;
		PORTG |= (0x01 << CH375_CS);
	PORTG |= (0x01 << CH375_WR);
	PORTG |= (0x01 << CH375_RD); 
	
	return( mData );
}

#define LED_INIT( )		{ PORTC |= 0x01; DDRC |= 0x01; }
#define LED_ON( )		{ PORTC &= 0xfe; }
#define LED_OFF( )	{ PORTC |= 0x01; }

void	mDelaymS( UINT8 ms )
{
	UINT16	i;
	while ( ms -- ) for ( i = 1000; i != 0; i -- ) mDelay1uS( );
}

void	mStopIfError( UINT8 iError )
{
	if ( iError == ERR_SUCCESS ) return;
	printf_P(PSTR("\r\n************************************************"));
	printf_P(PSTR("\r\nError: %02X\r\n"), (UINT16)iError);  /* G[\ */
	
	while ( 1 ) {
		LED_ON( );  /* LED_ */
		mDelaymS( 100 );
		LED_OFF( );
		mDelaymS( 100 );
	}
}

//FILE	uart_str = FDEV_SETUP_STREAM( uart_putchar, NULL, _FDEV_SETUP_WRITE );




int		main( ) 
{
	UINT8	i, c;
	UINT16	TotalCount;
	UINT8	*pCodeStr;

COM_Initial(MYUBRR);//VA|[g̃CjV

	

	printf_P(PSTR("Start\r\n" ));

		CH375_PORT_INIT( );
	LED_INIT( );
	LED_ON( );
	mDelaymS( 100 );
	LED_OFF( );

#if DISK_BASE_BUF_LEN == 0
	pDISK_BASE_BUF = &my_buffer[0];
#endif

	i = CH375LibInit( );
	mStopIfError( i );

	while ( 1 ) {
		printf_P(PSTR("Wait Udisk\r\n" ));
//		while ( CH375DiskStatus != DISK_CONNECT ) xQueryInterrupt( );
		while ( CH375DiskStatus < DISK_CONNECT ) {  /* USB}邱Ƃ҂Ă܂ */
			if ( CH375DiskConnect( ) == ERR_SUCCESS ) break;
			mDelaymS( 100 );
		}
		LED_ON( );
		mDelaymS( 200 );

		for ( i = 0; i < 10; i ++ ) {
			mDelaymS( 100 );
			printf_P(PSTR("Ready ?\r\n" ));
			if ( CH375DiskReady( ) == ERR_SUCCESS ) break;  /* USB memory is ready? */
		}

#if DISK_BASE_BUF_LEN
		if ( DISK_BASE_BUF_LEN < CH375vSectorSize ) { 
			printf_P(PSTR("Too large sector size\r\n" ));
			while ( CH375DiskConnect( ) == ERR_SUCCESS ) mDelaymS( 100 );
			continue;
		}
#endif
/* USB̃TCY */
/*		printf_P(PSTR("DiskSize\r\n" );
		i = CH375DiskSize( );  
		mStopIfError( i );
		printf_P(PSTR("TotalSize = %u MB \r\n", (unsigned int)( mCmdParam.DiskSize.mDiskSizeSec * (CH375vSectorSize/512) / 2048 ) );
*/

/* t@Cǂݍ */
		printf_P(PSTR("Open\r\n" ));
		strcpy( (char *)mCmdParam.Open.mPathName, "/C51/CH375HFT.C" );  /* t@C̖O */
		i = CH375FileOpen( );  /* t@CJ */
		if ( i == ERR_MISS_DIR || i == ERR_MISS_FILE ) {  /* t@C܂ */
/*tH_ t@C̃Xg */
			if ( i == ERR_MISS_DIR ) pCodeStr = (UINT8 *)"/*";  /* C51tH_Ȃ΁A[gtH_̑St@C */
			else pCodeStr = (UINT8 *)"/C51/CH375*";  /* C51tH_CH375Ƃt@Cl[ */
			printf_P(PSTR("List file %s\r\n"), pCodeStr );
			for ( c = 0; c < 254; c ++ ) { 
				strcpy( (char *)mCmdParam.Open.mPathName, (char *)pCodeStr );
				i = strlen( (char *)mCmdParam.Open.mPathName );
				mCmdParam.Open.mPathName[ i ] = c;
				i = CH375FileOpen( );
				if ( i == ERR_MISS_FILE ) break;  /* ̃t@CȂ */
				if ( i == ERR_FOUND_NAME ) {  /* ̃t@CT */
					printf_P(PSTR("  match file %03d#: %s\r\n"), (unsigned int)c, mCmdParam.Open.mPathName );
					continue;
				}
				else {
					mStopIfError( i );
					break;
				}
			}
		}
		else {
			mStopIfError( i );
			TotalCount = 600;  /* ǂݍރTCY */
			printf_P(PSTR("The begin %d charaters:\r\n"),TotalCount );
			while ( TotalCount ) {
				if ( TotalCount > MAX_BYTE_IO ) c = MAX_BYTE_IO;
				else c = TotalCount;
				mCmdParam.ByteRead.mByteCount = c;
				i = CH375ByteRead( );
				mStopIfError( i );
				TotalCount -= mCmdParam.ByteRead.mByteCount;
				for ( i=0; i!=mCmdParam.ByteRead.mByteCount; i++ ) printf_P(PSTR("%c"), mCmdParam.ByteRead.mByteBuffer[i] );
				if ( mCmdParam.ByteRead.mByteCount < c ) {
					printf_P(PSTR("\r\n" ));
					printf_P(PSTR("File end\r\n" ));
					break;
				}
			}
/*	    t@C̃|Cg̑
		mCmdParam.ByteLocate.mByteOffset = 608;
		CH375ByteLocate( );
		mCmdParam.ByteRead.mByteCount = 5;  //5oCgǂݍ
		CH375ByteRead( );   //t@C̃ItZbg608612܂ł5oCgǂݍށAn߂608oCgXLbv܂

		CH375FileOpen( );
		mCmdParam.ByteLocate.mByteOffset = 0xffffffff;  //|Cgt@C̍ŌɈړ]܂
		CH375ByteLocate( );
		mCmdParam.ByteWrite.mByteCount = 13;  //13oCg
		CH375ByteWrite( );
		mCmdParam.ByteWrite.mByteCount = 2;
		CH375ByteWrite( );
		mCmdParam.ByteWrite.mByteCount = 0;
		CH375ByteWrite( );     //0oCgȂAt@C̃TCYXV
*/
			printf_P(PSTR("Close\r\n" ));
			i = CH375FileClose( );
			mStopIfError( i );
		}

#ifdef EN_DISK_WRITE  /* USB̏ */
/* Vt@C */
		printf_P(PSTR("Create\r\n" ));
		strcpy( (char *)mCmdParam.Create.mPathName, "/NEWFILE.TXT" );  /* Vt@Cl[ */
		i = CH375FileCreate( );  /* Vt@CJ */
		mStopIfError( i );
		printf_P(PSTR("Write\r\n" ));
		pCodeStr = (UINT8 *)"Note: \xd\xãvOBYTETCYUSBǂݏ܂̂,600BytesRAM܂B\xd\xa";
		while( 1 ) {
			for ( i=0; i<MAX_BYTE_IO; i++ ) {
				c = *pCodeStr;
				mCmdParam.ByteWrite.mByteBuffer[i] = c;
				if ( c == 0 ) break;
				pCodeStr++;
			}
			if ( i == 0 ) break;
			mCmdParam.ByteWrite.mByteCount = i;
			i = CH375ByteWrite( );
			mStopIfError( i );
		}
/*t@C̃vpeB*/
/*0xff̏ꍇ́A܂*/
/*		printf_P(PSTR("Modify\r\n" );
		mCmdParam.Modify.mFileAttr = 0xff;
		mCmdParam.Modify.mFileTime = 0xffff;
		mCmdParam.Modify.mFileDate = MAKE_FILE_DATE( 2009, 3, 6 );
		mCmdParam.Modify.mFileSize = 0xffffffff;
		i = CH375FileModify( );
		mStopIfError( i );
*/
		printf_P(PSTR("Close\r\n" ));
		mCmdParam.Close.mUpdateLen = 1;  /* t@C̃TCYIɌvZ */
		i = CH375FileClose( );
		mStopIfError( i );

/* t@C̍폜 */
/*		printf_P(PSTR("Erase\r\n" ));
		strcpy( (char *)mCmdParam.Create.mPathName, "/OLD" );  //폜t@Cl[
		i = CH375FileErase( );  //t@C̍폜
		if ( i != ERR_SUCCESS ) printf_P(PSTR("Error: %02X\r\n", (UINT16)i );
*/

/* USB̏ */
/*		printf_P(PSTR("Disk\r\n" );
		i = CH375DiskQuery( );
		mStopIfError( i );
		printf_P(PSTR("Fat=%d, Total=%ld, Free=%ld\r\n", (UINT16)mCmdParam.Query.mDiskFat, mCmdParam.Query.mTotalSector, mCmdParam.Query.mFreeSector );
*/
#endif
		printf_P(PSTR("Take out\r\n" ));
//		while ( CH375DiskStatus != DISK_DISCONNECT ) xQueryInterrupt( );
		while ( CH375DiskStatus >= DISK_CONNECT ) {  /* USB𔲂̂҂Ă܂ */
			if ( CH375DiskConnect( ) != ERR_SUCCESS ) break;
			mDelaymS( 100 );
		}
		LED_OFF( );
		mDelaymS( 200 );
	}
}
